import argparse
import pathlib
import sys
from termcolor import colored
import colorama
from sys import platform as _platform

from cmpypass.db import create_db, remove_db
from cmpypass.CLI.handlers import user_handler, cat_handler, pwd_handler
from cmpypass.services.cache_svc import get_logged_user, remove_current_user_file_by_time
from cmpypass.helpers.route_helpers import db_file_route, get_current_file_route
from cmpypass.helpers.log_helpers import LogLevels, log

colorama.init()
__version__ = "0.0.1"

parser = argparse.ArgumentParser(prog='PYPASS')
parser.add_argument("--cdb", help="Creates the db", action='store_true')
parser.add_argument("--rdb", help="Removes the db", action='store_true')
parser.add_argument("--info", help="App info", action='store_true')
subparsers = parser.add_subparsers(dest="command")

user_parser = subparsers.add_parser(
    'u', help='Run u --help to see available options')
user_group = user_parser.add_mutually_exclusive_group()
user_group.add_argument(
    '--c', type=str, help='Create an user, requires the username as argument')
user_group.add_argument(
    '--login', type=str, help='Log into your user account, requires the username as argument')
user_group.add_argument('--logout', help='Log out', action="store_true")
user_group.add_argument(
    '--who', help='Checks the current user', action="store_true")
user_group.add_argument(
    '--l', help='List all the existing users', action="store_true")

category_parser = subparsers.add_parser(
    'cat', help='Run cat --help to see available options')
category_group = category_parser.add_mutually_exclusive_group()
category_group.add_argument(
    '--a', type=str, help='Adds a new category, requires the name as argument')
category_group.add_argument(
    '--d', type=str, help='Deletes a category, requires the name as argument')
category_group.add_argument(
    '--l', help='List all the available categories', action='store_true')

pwd_parser = subparsers.add_parser(
    'pwd', help='Run pwd --help to see available options')
pwd_group = pwd_parser.add_mutually_exclusive_group()
pwd_group.add_argument(
    '--a', type=str, help='Add a new password store, requires the username:category as argument')
pwd_group.add_argument(
    '--d', type=str, help='Delete a password store, requires the store name as argument')
pwd_group.add_argument(
    '--u', type=str, help='Update a password store, requires the store name as argument')
pwd_group.add_argument(
    '--la', help='List all your password records', action='store_true')
pwd_group.add_argument(
    '--l', help='List all your password records by category, requires the category name as argument')
pwd_group.add_argument('--cp', help='Copy the username or the password of the selected record. This command accepts 2 arguments, the name of the record and the part to copy. Example: ... --cp uname will copy the username, the other option is pwd', nargs=2)

'''
The optional 'id' argument in some functions is required for testing but will default to NoneType 
'''

# Checks if there's a current session and deletes it after 10min
remove_current_user_file_by_time(600)

logged_user = get_logged_user()

def main():
    args = parser.parse_args()

    if(args.info):
        output_log_info()
        sys.exit(0)

    db_exists = pathlib.Path(db_file_route).is_file()
    if not logged_user:
        if db_exists:
            if(args.cdb):
                print(
                    colored(f"Your db file is already created at {db_file_route}", "magenta"))
            elif(args.rdb):
                remove_db()
        else:
            if(args.cdb):
                create_db()
            elif(args.rdb):
                print(colored("Cannot remove the db file because it doesn't exists", "red"))
            else:
                print(f'The database is not created yet, try running {colored("--cdb", "magenta")}')
            sys.exit(0)
    else:
        if(args.cdb or args.rdb):
            print(colored("Operations over the db file can only be executed when not logged in, try pypass u --logout", "red"))
            sys.exit(0)

    try:
        if args.command is not None:
            if args.command == "u":
                user_handler.execute(args)
            else:
                if logged_user is not None:
                    if args.command == "cat":
                        cat_handler.execute(args)
                    elif args.command == "pwd":
                        pwd_handler.execute(args)
                    else:
                        print(colored("Command not recognized", "red"))
                else:
                    print(
                        f'You need to {colored("log in", "magenta")} in order to perform this operation')
    except KeyboardInterrupt:
        sys.exit(0)


def output_log_info():
    log(LogLevels.INFO, "Version", __version__)
    log(LogLevels.INFO, "Platform:", _platform)
    log(LogLevels.INFO, "Session file route:", get_current_file_route())
    log(LogLevels.INFO, "SQLite file route:", db_file_route)
    print("")

    log(LogLevels.WARN, "Make sure you have enabled permissions to write and read in both file directories, otherwise the app won't work")


if __name__ == "__main__":
    main()
