
from getpass import getpass
from sqlalchemy.orm.query import Query
from sqlalchemy.orm.session import Session
from sqlalchemy.sql.functions import user
from termcolor import colored
from tabulate import tabulate
import pyperclip
import sys

from cmpypass.db import Pypass, get_session
from cmpypass.services.cache_svc import get_logged_user
from cmpypass.services.encryption_svc import get_decrypted_password, get_encrypted_password
from cmpypass.services.category_svc import category_exists

logged_user = get_logged_user()

def get_pypass_and_category(s: Session, requested, id=None):
    user_id = id if id is not None else logged_user.id
    pypasses = s.query(Pypass).filter(Pypass.name == requested,
                                      (Pypass.user_id == user_id)).all()
    if len(pypasses) != 0:
        if len(pypasses) > 1:
            print(
                "This store exists in different categories, write the one you want to remove:")
            categories = []
            for pypass in pypasses:
                categories.append(pypass.category)
                print(colored(f"- {pypass.category}", "magenta"))
            target_category = input("")
            if target_category not in categories:
                print(colored("Invalid category", "red"))
                s.close()
                sys.exit(0)
        else:
            target_category = pypasses[0].category
        return {
            "category": target_category,
            "pypass": pypasses[0]
        }
    else:
        print(colored(
            f"Password store with name {requested} does not exist in any category", "red"))
        s.close()
        sys.exit(0)


def add_pypass(name, username, category, password, id=None):
    user_id = id if id is not None else logged_user.id
    s: Session = get_session()
    if category_exists(category, user_id):
        existant_paypass = s.query(Pypass).filter(
            Pypass.name == name, (Pypass.category == category), (Pypass.user_id == user_id)).first()
        if existant_paypass is None:
            pwd = Pypass(name, username, get_encrypted_password(
                password), category, user_id)
            s.add(pwd)
            s.commit()
            print(
                f'Password store {colored(name, "magenta")} {colored("succesfully created", "green")} and added to {colored(category, "magenta")}')
            return
        print(colored(
            f"Password store with name {name} and category {category} already exists, try updating the record instead", "red"))
    else:
        print(colored(f"Category {category} does not exist", "red"))
    s.close()


def delete_pypass(name, id=None):
    user_id = id if id is not None else logged_user.id
    target_category = None
    s: Session = get_session()
    target_category = get_pypass_and_category(s, name, user_id)["category"]
    s.query(Pypass).filter(Pypass.name == name, (Pypass.category ==
                                                 target_category), (Pypass.user_id == user_id)).delete()
    s.commit()
    s.close()
    print(
        f'Password store with name {colored(name, "magenta")} {colored("succesfully deleted", "green")} from category {colored(target_category, "magenta")}')


def update_pypass(name):
    s: Session = get_session()
    data = get_pypass_and_category(s, name)
    target_category = data["category"]
    pypass: Pypass = data["pypass"]

    print(colored(
        "Select a new name for the store: (Empty to keep the old one)", "magenta"))
    store_name = input("> ")
    print(colored(
        "Select a new username for the store: (Empty to keep the old one)", "magenta"))
    username = input("> ")
    print(colored(
        "Select a new password for the store: (Empty to keep the old one)", "magenta"))
    password = getpass("> ")

    store_name = pypass.name if store_name == "" else store_name
    username = pypass.username if username == "" else username
    password = pypass.password if password == "" else get_encrypted_password(
        password)

    s.query(Pypass).filter(Pypass.name == name, (Pypass.category == target_category), (Pypass.user_id == logged_user.id)).update(
        {Pypass.name: store_name, Pypass.password: password, Pypass.username: username})
    s.commit()
    s.close()
    print(
        f'Password store with name {colored(name, "magenta")} {colored("succesfully updated", "green")} from category {colored(target_category, "magenta")}')


def copy_pypass(name, part, id=None):
    user_id = id if id is not None else logged_user.id
    s: Session = get_session()
    selected = get_pypass_and_category(s, name, user_id)["pypass"]
    try:
        if part == "uname":
            pyperclip.copy(selected.username)
            print(colored("Username copied to clipboard!", "green"))
            return
        pyperclip.copy(get_decrypted_password(selected.password))
        print(colored("Password copied to clipboard!", "green"))
    except pyperclip.PyperclipException:
        print(colored(
            f"Couldn't copy to the clipboard, if you are on a Linux distro try sudo apt install xclip", "red"))
    finally:
        s.close()

def list_pypasses():
    s: Session = get_session()
    pypasses = s.query(Pypass).filter(Pypass.user_id == logged_user.id).order_by(
        Pypass.category, Pypass.name, Pypass.username)
    s.close()
    table_list = []
    for pypass in pypasses:
        row = [pypass.id, pypass.name, pypass.username, pypass.category]
        table_list.append(row)
    print(tabulate(table_list, headers=[
          "id", "name", "username", "category"], tablefmt="github"))


def list_pypasses_by_category(category):
    s: Session = get_session()
    if category_exists(category):
        pypasses = s.query(Pypass).filter(Pypass.category == category, (
            Pypass.user_id == logged_user.id)).order_by(Pypass.name, Pypass.username).all()
        table_list = []
        for pypass in pypasses:
            row = [pypass.id, pypass.name, pypass.username]
            table_list.append(row)
        print(tabulate(table_list, headers=[
              "id", "name", "username"], tablefmt="github"))
        return
    s.close()
    print(colored(f"Category {category} does not exist", "red"))
